@@ -1,5 +1,7 @@
 ##############################################
 package PDL::PP::PdlDimsObj; # Hold more dims
+use strict;
+use warnings;
 use Carp;
 
 sub new {
@@ -8,20 +10,21 @@ sub new {
 }
 
 sub get_indobj_make {
-	my($this,$expr) = @_;
-	$expr =~ /^([a-zA-Z0-9]+)(?:=([0-9]+))?$/ or confess "Invalid index expr '$expr'\n";
-	my $name = $1; my $val = $2;
-	my $indobj;
-	if(defined $this->{$name}) {
-		$indobj = $this->{$name};
-	} else {
-		$indobj = PDL::PP::Ind->new($name);
-		$this->{$name}=$indobj;
-	}
-	if(defined $val) { $indobj->add_value($val); }
-	return $indobj;
+  my ($this,$expr,$calc) = @_;
+  my ($name, $val) = $expr =~ /^([a-zA-Z0-9]+)(?:=([0-9]+))?$/ or confess "Invalid index expr '$expr'\n";
+  confess "Error: both simple value '$val' and CALC '$calc'"
+    if $calc && defined $val;
+  $val //= $calc;
+  my $indobj = $this->{$name} //= PDL::PP::Ind->new($name);
+  $indobj->add_value($val) if defined $val;
+  return $indobj;
 }
 
+sub ind_obj {$_[0]{$_[1]}}
+sub ind_names {keys %{$_[0]}}
+sub ind_fromcomp {grep defined $_->{From}, values %{$_[0]}}
+sub ind_notfromcomp {grep !defined $_->{From}, values %{$_[0]}}
+
 #####################################################################
 #
 # Encapsulate one index.
@@ -34,11 +37,11 @@ sub new {
 	bless {Name => $name},$type;
 }
 
-# set the value of an index, also used by perl level threading
+# set the value of an index, also used by perl level broadcasting
 sub add_value {
 	my($this,$val) = @_;
 	croak("index values for $this->{Name} must be positive")
-	  unless $val > 0;
+	  if $val =~ /^\d+$/ and $val <= 0;
 	return $this->{Value} = $val if
 		!defined $this->{Value} or
 		$this->{Value} == -1 or
@@ -51,14 +54,14 @@ sub set_from { my($this,$otherpar) = @_;
 	$this->{From} = $otherpar;
 }
 
-sub name {$_[0]->{Name}}
+sub name {$_[0]{Name}}
 
 # where it occurs in the C arrays that track it (at least name and size)
 sub set_index {
   my ($this, $i) = @_;
   $this->{Index} = $i;
 }
-sub get_index {$_[0]->{Index}}
+sub get_index {$_[0]{Index} // confess "unknown index for $_[0]{Name}"}
 
 sub get_initdim { my($this) = @_;
 	my $init = $this->{Value} //
